/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation (Xerox). 
 * Portions created by Xerox are Copyright (C) 1999-2001 Xerox Corporation.
 * Portions created by Palo Alto Research Center, Incorporated are 
 * Copyright (C) 1999-2002 Palo Alto Research Center, Incorporated.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.WindowEvent;

import javax.swing.*;

import org.aspectj.ajde.Ajde;
import org.netbeans.core.actions.MemoryMeterAction;
import org.openide.awt.Actions;
import org.openide.explorer.ExplorerPanel;
import org.openide.filesystems.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.*;
import org.openide.windows.*;

/** Action to enable/disable the Aspectj browser in the Netbeans explorer.
 *
 * @author  Phil Sager
 */
public class AJStartBrowserAction extends BooleanStateAction {
    
    private boolean installed = false;
    private boolean firstInstall = true;
    private Actions.ToolbarToggleButton  presenter = null;
    
    public StructureViewTab structureViewTab = null;
    
    /**
     *  TODO: the toolbar buttons are not enabled the first time the AJ enable
     *  button is pressed. Use cookies to enable/disable them.
     */
    public void performAction() {
    	try {
	        Workspace editing = org.openide.TopManager.getDefault()
	                                                  .getWindowManager()
	                                                  .findWorkspace("Editing"); //NOI18N
	        
	        if (editing != null) {
	            editing.activate();
	        }
	        CallableSystemAction BUILD = (CallableSystemAction)SystemAction.get(AJBuildAction.class);        
	        CallableSystemAction SELECT = (CallableSystemAction)SystemAction.get(AJSelectConfigurationAction.class); 
	        //CallableSystemAction RUN = (CallableSystemAction)SystemAction.get(AJRunAction.class);
	        CallableSystemAction OPTIONS = (CallableSystemAction)SystemAction.get(AJOptionsAction.class);
	        
	        //BUILD.set
	        MemoryMeterAction action;
	        
	        if (!installed) {
	            BUILD.setEnabled(true);
	            SELECT.setEnabled(true);
	            OPTIONS.setEnabled(true);
	
	            NbManager.init();
	            structureViewTab = new StructureViewTab(NbManager.INSTANCE.getViewPanel());
	            
	            org.openide.TopManager.getDefault()
	                                  .getWindowManager()
	                                  .getMainWindow()
	                                  .addWindowListener(
	                             new java.awt.event.WindowAdapter() {
	                                  public void windowDeiconified(WindowEvent e) {
	                    }
	                });
	            Runnable update = new Runnable() {
	                public void run() {
	                    toggleStructureView(true);
	                    structureViewTab.setVisible(true);
	                }
	            };
	            if (SwingUtilities.isEventDispatchThread()) {
	                update.run();
	            } else {
	                try {
	                    SwingUtilities.invokeAndWait(update);
	                } catch (Exception e) {}
	            }
	            installed = true;
	            //AjdeUIManager.getDefault().getViewManager().updateView();
	        } else {
	            BUILD.setEnabled(false);
	            SELECT.setEnabled(false);
	            OPTIONS.setEnabled(false);
	            
	            toggleStructureView(false);
	            installed = false;
	        }
	        if (firstInstall) {
	            org.openide.TopManager.getDefault().getRepository()
	                                               .addRepositoryListener(new RepositoryListener() {
	                public void fileSystemAdded(RepositoryEvent ev) {
	                    //AjdeUIManager.getDefault().getViewManager().updateConfigsList();
	                }
	                public void fileSystemRemoved(RepositoryEvent ev) {
	                    //AjdeUIManager.getDefault().getViewManager().updateConfigsList();
	                }
	                public void fileSystemPoolReordered(RepositoryReorderedEvent ev) {
	                    // ignored
	                }
	            });
	            firstInstall = false;
	        }
    	} catch (Throwable t) {
    		Ajde.getDefault().getErrorHandler().handleError("Could not start AJDE.", t);	
    	}
    }

    /** Get the current state.
    * @return <code>true</code> if on
    */
    public boolean getBooleanState() {
        return installed;
    }

    /** Set the current state.
    * Fires a change event, which should be used to affect other components when
    * its state is toggled.
    * @param value <code>true</code> to turn on, <code>false</code> to turn off
    */
    public void setBooleanState(boolean value) {
        performAction();
        //if (!installed && value) {
        //	
        //}
    }

    public java.awt.Component getToolbarPresenter() {
        if (presenter == null) {
            presenter = new Actions.ToolbarToggleButton (this);
        }    
        return presenter;
    }
    
    public String getName() {
        return NbBundle.getMessage(AJStartBrowserAction.class, "LBL_StartBrowserAction");
    }
    
    protected String iconResource() {
        return "/org/aspectj/ajde/resources/actions/startAjde.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (StartBrowserAction.class);
    }
    
    /**
     *  Display/Hide the Aspectj Browser window in the Explorer. This method docks
     *  the AJ Browser window into the main Explorer tabbed pane window, allowing the
     *  user to change Explorer views without disabling AJ...pjs
     *
     *  @param show boolean value. true == show, which opens the AJ Browser window.
     */
    private void toggleStructureView(boolean show) {
        try {
            Mode ajExplore = getExplorerMode();
            ajExplore.dockInto(structureViewTab);
            
            if (show) {
                structureViewTab.open();
                structureViewTab.setName("Aspectj");

            } else {
                structureViewTab.close();

                }
            
        } catch (Exception e) {
        	Ajde.getDefault().getErrorHandler().handleError("Could not toggle structure view.", e);
        }
    }

    /**
     *  Finds the Mode for the main Explorer window. This method uses pre-defined 
     *  default mode name. These modes are defined in the core, however, they are 
     *  not dependable, as they are not part of the APIs....pjs
     *  TODO: throw exception on mode == null which means there is no Explorer panel.
     */
    private static Mode getExplorerMode(){
        Mode mode = null;
        Workspace[] workspaces = org.openide.TopManager.getDefault()
                                                       .getWindowManager()
                                                       .getWorkspaces();

        for (int i = 0; i < workspaces.length; i++){
            if(workspaces[i].getName().equals("Editing")){
                mode = workspaces[i].findMode("explorer");
                return mode;
            }
        }
        return mode; // null...
    }
    
    /** Perform extra initialization of this action's singleton. 
     * PLEASE do not use constructors for this purpose! */
      protected void initialize () {
          super.initialize();
            super.setEnabled(true);
      }


static class StructureViewComponent extends TopComponent {
    public StructureViewComponent(JPanel panel) {
        this.setName("AspectJ Browser");
        this.setLayout(new BorderLayout());
        this.add(panel, BorderLayout.CENTER);
        this.add(panel);
    }
    
    public Image getIcon() {
        return null; //((ImageIcon)ForteIcons.INSTANCE.getStartAjdeIcon()).getImage();
    }
    
    public SystemAction[] getSystemActions() {
        return new SystemAction[0];
    }
}  

static class StructureViewTab extends ExplorerPanel {

    public StructureViewTab() {
    	super();	
    }

    public StructureViewTab(JPanel panel) {
        this.setLayout(new BorderLayout());
        this.add(panel, BorderLayout.CENTER);
    } 

    
    public Image getIcon() {
        return ((ImageIcon)NbIconRegistry.INSTANCE.getStartAjdeIcon()).getImage();
    }
}
}
